@@ -1,17 +1,28 @@
-from openhands.events.observation.agent import AgentStateChangedObservation
+from openhands.events.event import RecallType
+from openhands.events.observation.agent import (
+    AgentCondensationObservation,
+    AgentStateChangedObservation,
+    AgentThinkObservation,
+    RecallObservation,
+)
 from openhands.events.observation.browse import BrowserOutputObservation
 from openhands.events.observation.commands import (
+    CmdOutputMetadata,
     CmdOutputObservation,
     IPythonRunCellObservation,
 )
 from openhands.events.observation.delegate import AgentDelegateObservation
-from openhands.events.observation.empty import NullObservation
+from openhands.events.observation.empty import (
+    NullObservation,
+)
 from openhands.events.observation.error import ErrorObservation
+from openhands.events.observation.file_download import FileDownloadObservation
 from openhands.events.observation.files import (
     FileEditObservation,
     FileReadObservation,
     FileWriteObservation,
 )
+from openhands.events.observation.mcp import MCPObservation
 from openhands.events.observation.observation import Observation
 from openhands.events.observation.reject import UserRejectObservation
 from openhands.events.observation.success import SuccessObservation
@@ -19,7 +30,9 @@ from openhands.events.observation.success import SuccessObservation
 __all__ = [
     'Observation',
     'NullObservation',
+    'AgentThinkObservation',
     'CmdOutputObservation',
+    'CmdOutputMetadata',
     'IPythonRunCellObservation',
     'BrowserOutputObservation',
     'FileReadObservation',
@@ -30,4 +43,9 @@ __all__ = [
     'AgentDelegateObservation',
     'SuccessObservation',
     'UserRejectObservation',
+    'AgentCondensationObservation',
+    'RecallObservation',
+    'RecallType',
+    'MCPObservation',
+    'FileDownloadObservation',
 ]
