@@ -3,6 +3,7 @@ import base64
 import re
 import zlib
 from collections import namedtuple
+from typing import Iterable
 
 from . import exceptions
 from .snippets import findall_regex, split_by_regex
@@ -24,7 +25,7 @@ unified_header_index = re.compile('^Index: (.+)$')
 unified_header_old_line = re.compile(r'^--- ' + file_timestamp_str + '$')
 unified_header_new_line = re.compile(r'^\+\+\+ ' + file_timestamp_str + '$')
 unified_hunk_start = re.compile(r'^@@ -(\d+),?(\d*) \+(\d+),?(\d*) @@(.*)$')
-unified_change = re.compile('^([-+ ])(.*)$')
+unified_change = re.compile('^([-+ ])(.*)$', re.MULTILINE)
 
 context_header_old_line = re.compile(r'^\*\*\* ' + file_timestamp_str + '$')
 context_header_new_line = re.compile('^--- ' + file_timestamp_str + '$')
@@ -71,11 +72,8 @@ cvs_header_timestamp_colon = re.compile(r':([\d.]+)\t(.+)')
 old_cvs_diffcmd_header = re.compile('^diff.* (.+):(.*) (.+):(.*)$')
 
 
-def parse_patch(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_patch(text: str | list[str]) -> Iterable[diffobj]:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     # maybe use this to nuke all of those line endings?
     # lines = [x.splitlines()[0] for x in lines]
@@ -104,18 +102,15 @@ def parse_patch(text):
             yield diffobj(header=h, changes=d, text=difftext)
 
 
-def parse_header(text):
+def parse_header(text: str | list[str]) -> header | None:
     h = parse_scm_header(text)
     if h is None:
         h = parse_diff_header(text)
     return h
 
 
-def parse_scm_header(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_scm_header(text: str | list[str]) -> header | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     check = [
         (git_header_index, parse_git_header),
@@ -154,11 +149,8 @@ def parse_scm_header(text):
     return None
 
 
-def parse_diff_header(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_diff_header(text: str | list[str]) -> header | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     check = [
         (unified_header_new_line, parse_unified_header),
@@ -178,10 +170,10 @@ def parse_diff_header(text):
     return None  # no header?
 
 
-def parse_diff(text):
-    try:
+def parse_diff(text: str | list[str]) -> list[Change] | None:
+    if isinstance(text, str):
         lines = text.splitlines()
-    except AttributeError:
+    else:
         lines = text
 
     check = [
@@ -200,11 +192,8 @@ def parse_diff(text):
     return None
 
 
-def parse_git_header(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_git_header(text: str | list[str]) -> header | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     old_version = None
     new_version = None
@@ -225,7 +214,7 @@ def parse_git_header(text):
             new_version = g.group(2)
             continue
 
-        # git always has it's own special headers
+        # git always has its own special headers
         o = git_header_old_line.match(line)
         if o:
             old_path = o.group(1)
@@ -275,11 +264,8 @@ def parse_git_header(text):
     return None
 
 
-def parse_svn_header(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_svn_header(text: str | list[str]) -> header | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     headers = findall_regex(lines, svn_header_index)
     if len(headers) == 0:
@@ -346,11 +332,8 @@ def parse_svn_header(text):
     return None
 
 
-def parse_cvs_header(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_cvs_header(text: str | list[str]) -> header | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     headers = findall_regex(lines, cvs_header_rcs)
     headers_old = findall_regex(lines, old_cvs_diffcmd_header)
@@ -430,11 +413,8 @@ def parse_cvs_header(text):
     return None
 
 
-def parse_diffcmd_header(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_diffcmd_header(text: str | list[str]) -> header | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     headers = findall_regex(lines, diffcmd_header)
     if len(headers) == 0:
@@ -454,11 +434,8 @@ def parse_diffcmd_header(text):
     return None
 
 
-def parse_unified_header(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_unified_header(text: str | list[str]) -> header | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     headers = findall_regex(lines, unified_header_new_line)
     if len(headers) == 0:
@@ -490,11 +467,8 @@ def parse_unified_header(text):
     return None
 
 
-def parse_context_header(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_context_header(text: str | list[str]) -> header | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     headers = findall_regex(lines, context_header_old_line)
     if len(headers) == 0:
@@ -526,11 +500,8 @@ def parse_context_header(text):
     return None
 
 
-def parse_default_diff(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_default_diff(text: str | list[str]) -> list[Change] | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     old = 0
     new = 0
@@ -582,11 +553,8 @@ def parse_default_diff(text):
     return None
 
 
-def parse_unified_diff(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_unified_diff(text: str | list[str]) -> list[Change] | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     old = 0
     new = 0
@@ -606,38 +574,45 @@ def parse_unified_diff(text):
             h = unified_hunk_start.match(hunk[0])
             del hunk[0]
             if h:
-                old = int(h.group(1))
-                if len(h.group(2)) > 0:
-                    old_len = int(h.group(2))
-                else:
-                    old_len = 0
+                # The hunk header @@ -1,6 +1,6 @@ means:
+                # - Start at line 1 in the old file and show 6 lines
+                # - Start at line 1 in the new file and show 6 lines
+                old = int(h.group(1))  # Starting line in old file
+                old_len = (
+                    int(h.group(2)) if len(h.group(2)) > 0 else 1
+                )  # Number of lines in old file
 
-                new = int(h.group(3))
-                if len(h.group(4)) > 0:
-                    new_len = int(h.group(4))
-                else:
-                    new_len = 0
+                new = int(h.group(3))  # Starting line in new file
+                new_len = (
+                    int(h.group(4)) if len(h.group(4)) > 0 else 1
+                )  # Number of lines in new file
 
                 h = None
                 break
 
+        # Process each line in the hunk
         for n in hunk:
-            c = unified_change.match(n)
-            if c:
-                kind = c.group(1)
-                line = c.group(2)
+            # Each line in a unified diff starts with a space (context), + (addition), or - (deletion)
+            # The first character is the kind, the rest is the line content
+            kind = (
+                n[0] if len(n) > 0 else ' '
+            )  # Empty lines in the hunk are treated as context lines
+            line = n[1:] if len(n) > 1 else ''
 
-                if kind == '-' and (r != old_len or r == 0):
-                    changes.append(Change(old + r, None, line, hunk_n))
-                    r += 1
-                elif kind == '+' and (i != new_len or i == 0):
-                    changes.append(Change(None, new + i, line, hunk_n))
-                    i += 1
-                elif kind == ' ':
-                    if r != old_len and i != new_len:
-                        changes.append(Change(old + r, new + i, line, hunk_n))
-                    r += 1
-                    i += 1
+            # Process the line based on its kind
+            if kind == '-' and (r != old_len or r == 0):
+                # Line was removed from the old file
+                changes.append(Change(old + r, None, line, hunk_n))
+                r += 1
+            elif kind == '+' and (i != new_len or i == 0):
+                # Line was added in the new file
+                changes.append(Change(None, new + i, line, hunk_n))
+                i += 1
+            elif kind == ' ':
+                # Context line - exists in both old and new file
+                changes.append(Change(old + r, new + i, line, hunk_n))
+                r += 1
+                i += 1
 
     if len(changes) > 0:
         return changes
@@ -645,11 +620,8 @@ def parse_unified_diff(text):
     return None
 
 
-def parse_context_diff(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_context_diff(text: str | list[str]) -> list[Change] | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     old = 0
     new = 0
@@ -788,11 +760,8 @@ def parse_context_diff(text):
     return None
 
 
-def parse_ed_diff(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_ed_diff(text: str | list[str]) -> list[Change] | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     old = 0
     j = 0
@@ -871,12 +840,9 @@ def parse_ed_diff(text):
     return None
 
 
-def parse_rcs_ed_diff(text):
+def parse_rcs_ed_diff(text: str | list[str]) -> list[Change] | None:
     # much like forward ed, but no 'c' type
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+    lines = text.splitlines() if isinstance(text, str) else text
 
     old = 0
     j = 0
@@ -898,7 +864,7 @@ def parse_rcs_ed_diff(text):
 
                 hunk_kind = o.group(1)
                 old = int(o.group(2))
-                size = int(o.group(3))
+                size = int(o.group(3)) if o.group(3) else 0
 
                 if hunk_kind == 'a':
                     old += total_change_size + 1
@@ -919,15 +885,11 @@ def parse_rcs_ed_diff(text):
 
     if len(changes) > 0:
         return changes
-
     return None
 
 
-def parse_git_binary_diff(text):
-    try:
-        lines = text.splitlines()
-    except AttributeError:
-        lines = text
+def parse_git_binary_diff(text: str | list[str]) -> list[Change] | None:
+    lines = text.splitlines() if isinstance(text, str) else text
 
     changes: list[Change] = list()
 
