@@ -5,18 +5,24 @@ import shutil
 import subprocess
 
 import jinja2
-import requests
+from pydantic import SecretStr
 
 from openhands.core.config import LLMConfig
 from openhands.core.logger import openhands_logger as logger
+from openhands.integrations.service_types import ProviderType
 from openhands.llm.llm import LLM
-from openhands.resolver.github_issue import GithubIssue
+from openhands.resolver.interfaces.bitbucket import BitbucketIssueHandler
+from openhands.resolver.interfaces.github import GithubIssueHandler
+from openhands.resolver.interfaces.gitlab import GitlabIssueHandler
+from openhands.resolver.interfaces.issue import Issue
+from openhands.resolver.interfaces.issue_definitions import ServiceContextIssue
 from openhands.resolver.io_utils import (
-    load_all_resolver_outputs,
     load_single_resolver_output,
 )
 from openhands.resolver.patching import apply_diff, parse_patch
 from openhands.resolver.resolver_output import ResolverOutput
+from openhands.resolver.utils import identify_token
+from openhands.utils.async_utils import GENERAL_TIMEOUT, call_async_from_sync
 
 
 def apply_patch(repo_dir: str, patch: str) -> None:
@@ -29,7 +35,7 @@ def apply_patch(repo_dir: str, patch: str) -> None:
     diffs = parse_patch(patch)
     for diff in diffs:
         if not diff.header.new_path:
-            print('Warning: Could not determine file to patch')
+            logger.warning('Could not determine file to patch')
             continue
 
         # Remove both "a/" and "b/" prefixes from paths
@@ -49,7 +55,7 @@ def apply_patch(repo_dir: str, patch: str) -> None:
             assert old_path is not None
             if os.path.exists(old_path):
                 os.remove(old_path)
-                print(f'Deleted file: {old_path}')
+                logger.info(f'Deleted file: {old_path}')
             continue
 
         # Handle file rename
@@ -99,7 +105,7 @@ def apply_patch(repo_dir: str, patch: str) -> None:
             split_content = []
 
         if diff.changes is None:
-            print(f'Warning: No changes to apply for {old_path}')
+            logger.warning(f'No changes to apply for {old_path}')
             continue
 
         new_content = apply_diff(diff, split_content)
@@ -112,7 +118,7 @@ def apply_patch(repo_dir: str, patch: str) -> None:
             for line in new_content:
                 print(line, file=f)
 
-    print('Patch applied successfully')
+    logger.info('Patch applied successfully')
 
 
 def initialize_repo(
@@ -136,7 +142,7 @@ def initialize_repo(
         shutil.rmtree(dest_dir)
 
     shutil.copytree(src_dir, dest_dir)
-    print(f'Copied repository to {dest_dir}')
+    logger.info(f'Copied repository to {dest_dir}')
 
     # Checkout the base commit if provided
     if base_commit:
@@ -147,13 +153,13 @@ def initialize_repo(
             text=True,
         )
         if result.returncode != 0:
-            print(f'Error checking out commit: {result.stderr}')
+            logger.info(f'Error checking out commit: {result.stderr}')
             raise RuntimeError('Failed to check out commit')
 
     return dest_dir
 
 
-def make_commit(repo_dir: str, issue: GithubIssue, issue_type: str) -> None:
+def make_commit(repo_dir: str, issue: Issue, issue_type: str) -> None:
     """Make a commit with the changes to the repository.
 
     Args:
@@ -178,14 +184,14 @@ def make_commit(repo_dir: str, issue: GithubIssue, issue_type: str) -> None:
             shell=True,
             check=True,
         )
-        print('Git user configured as openhands')
+        logger.info('Git user configured as openhands')
 
     # Add all changes to the git index
     result = subprocess.run(
         f'git -C {repo_dir} add .', shell=True, capture_output=True, text=True
     )
     if result.returncode != 0:
-        print(f'Error adding files: {result.stderr}')
+        logger.error(f'Error adding files: {result.stderr}')
         raise RuntimeError('Failed to add files to git')
 
     # Check the status of the git index
@@ -198,7 +204,9 @@ def make_commit(repo_dir: str, issue: GithubIssue, issue_type: str) -> None:
 
     # If there are no changes, raise an error
     if not status_result.stdout.strip():
-        print(f'No changes to commit for issue #{issue.number}. Skipping commit.')
+        logger.error(
+            f'No changes to commit for issue #{issue.number}. Skipping commit.'
+        )
         raise RuntimeError('ERROR: Openhands failed to make code changes.')
 
     # Prepare the commit message
@@ -214,266 +222,223 @@ def make_commit(repo_dir: str, issue: GithubIssue, issue_type: str) -> None:
         raise RuntimeError(f'Failed to commit changes: {result}')
 
 
-def branch_exists(base_url: str, branch_name: str, headers: dict) -> bool:
-    """Check if a branch exists in the GitHub repository.
-
-    Args:
-        base_url: The base URL of the GitHub repository API
-        branch_name: The name of the branch to check
-        headers: The HTTP headers to use for authentication
-    """
-    print(f'Checking if branch {branch_name} exists...')
-    response = requests.get(f'{base_url}/branches/{branch_name}', headers=headers)
-    exists = response.status_code == 200
-    print(f'Branch {branch_name} exists: {exists}')
-    return exists
-
-
 def send_pull_request(
-    github_issue: GithubIssue,
-    github_token: str,
-    github_username: str | None,
+    issue: Issue,
+    token: str,
+    username: str | None,
+    platform: ProviderType,
     patch_dir: str,
     pr_type: str,
     fork_owner: str | None = None,
     additional_message: str | None = None,
     target_branch: str | None = None,
     reviewer: str | None = None,
+    pr_title: str | None = None,
+    base_domain: str | None = None,
 ) -> str:
-    """Send a pull request to a GitHub repository.
+    """Send a pull request to a GitHub, GitLab, or Bitbucket repository.
 
     Args:
-        github_issue: The issue to send the pull request for
-        github_token: The GitHub token to use for authentication
-        github_username: The GitHub username, if provided
+        issue: The issue to send the pull request for
+        token: The token to use for authentication
+        username: The username, if provided
+        platform: The platform of the repository.
         patch_dir: The directory containing the patches to apply
         pr_type: The type: branch (no PR created), draft or ready (regular PR created)
         fork_owner: The owner of the fork to push changes to (if different from the original repo owner)
         additional_message: The additional messages to post as a comment on the PR in json list format
         target_branch: The target branch to create the pull request against (defaults to repository default branch)
+        reviewer: The username of the reviewer to assign
+        pr_title: Custom title for the pull request (optional)
+        base_domain: The base domain for the git server (defaults to "github.com" for GitHub, "gitlab.com" for GitLab, and "bitbucket.org" for Bitbucket)
     """
     if pr_type not in ['branch', 'draft', 'ready']:
         raise ValueError(f'Invalid pr_type: {pr_type}')
 
-    # Set up headers and base URL for GitHub API
-    headers = {
-        'Authorization': f'token {github_token}',
-        'Accept': 'application/vnd.github.v3+json',
-    }
-    base_url = f'https://api.github.com/repos/{github_issue.owner}/{github_issue.repo}'
+    # Determine default base_domain based on platform
+    if base_domain is None:
+        if platform == ProviderType.GITHUB:
+            base_domain = 'github.com'
+        elif platform == ProviderType.GITLAB:
+            base_domain = 'gitlab.com'
+        else:  # platform == ProviderType.BITBUCKET
+            base_domain = 'bitbucket.org'
+
+    # Create the appropriate handler based on platform
+    handler = None
+    if platform == ProviderType.GITHUB:
+        handler = ServiceContextIssue(
+            GithubIssueHandler(issue.owner, issue.repo, token, username, base_domain),
+            None,
+        )
+    elif platform == ProviderType.GITLAB:
+        handler = ServiceContextIssue(
+            GitlabIssueHandler(issue.owner, issue.repo, token, username, base_domain),
+            None,
+        )
+    elif platform == ProviderType.BITBUCKET:
+        handler = ServiceContextIssue(
+            BitbucketIssueHandler(
+                issue.owner, issue.repo, token, username, base_domain
+            ),
+            None,
+        )
+    else:
+        raise ValueError(f'Unsupported platform: {platform}')
 
     # Create a new branch with a unique name
-    base_branch_name = f'openhands-fix-issue-{github_issue.number}'
-    branch_name = base_branch_name
-    attempt = 1
-
-    # Find a unique branch name
-    print('Checking if branch exists...')
-    while branch_exists(base_url, branch_name, headers):
-        attempt += 1
-        branch_name = f'{base_branch_name}-try{attempt}'
+    base_branch_name = f'openhands-fix-issue-{issue.number}'
+    branch_name = handler.get_branch_name(
+        base_branch_name=base_branch_name,
+    )
 
     # Get the default branch or use specified target branch
-    print('Getting base branch...')
+    logger.info('Getting base branch...')
     if target_branch:
         base_branch = target_branch
-        # Verify the target branch exists
-        response = requests.get(f'{base_url}/branches/{target_branch}', headers=headers)
-        if response.status_code != 200:
+        exists = handler.branch_exists(branch_name=target_branch)
+        if not exists:
             raise ValueError(f'Target branch {target_branch} does not exist')
     else:
-        response = requests.get(f'{base_url}', headers=headers)
-        response.raise_for_status()
-        base_branch = response.json()['default_branch']
-    print(f'Base branch: {base_branch}')
+        base_branch = handler.get_default_branch_name()
+    logger.info(f'Base branch: {base_branch}')
 
     # Create and checkout the new branch
-    print('Creating new branch...')
+    logger.info('Creating new branch...')
     result = subprocess.run(
         ['git', '-C', patch_dir, 'checkout', '-b', branch_name],
         capture_output=True,
         text=True,
     )
     if result.returncode != 0:
-        print(f'Error creating new branch: {result.stderr}')
+        logger.error(f'Error creating new branch: {result.stderr}')
         raise RuntimeError(
             f'Failed to create a new branch {branch_name} in {patch_dir}:'
         )
 
     # Determine the repository to push to (original or fork)
-    push_owner = fork_owner if fork_owner else github_issue.owner
-    push_repo = github_issue.repo
+    push_owner = fork_owner if fork_owner else issue.owner
 
-    print('Pushing changes...')
-    username_and_token = (
-        f'{github_username}:{github_token}'
-        if github_username
-        else f'x-auth-token:{github_token}'
-    )
-    push_url = f'https://{username_and_token}@github.com/{push_owner}/{push_repo}.git'
+    handler._strategy.set_owner(push_owner)
+
+    logger.info('Pushing changes...')
+    push_url = handler.get_clone_url()
     result = subprocess.run(
         ['git', '-C', patch_dir, 'push', push_url, branch_name],
         capture_output=True,
         text=True,
     )
     if result.returncode != 0:
-        print(f'Error pushing changes: {result.stderr}')
+        logger.error(f'Error pushing changes: {result.stderr}')
         raise RuntimeError('Failed to push changes to the remote repository')
 
     # Prepare the PR data: title and body
-    pr_title = f'Fix issue #{github_issue.number}: {github_issue.title}'
-    pr_body = f'This pull request fixes #{github_issue.number}.'
+    final_pr_title = (
+        pr_title if pr_title else f'Fix issue #{issue.number}: {issue.title}'
+    )
+    pr_body = f'This pull request fixes #{issue.number}.'
     if additional_message:
         pr_body += f'\n\n{additional_message}'
     pr_body += '\n\nAutomatic fix generated by [OpenHands](https://github.com/All-Hands-AI/OpenHands/) 🙌'
 
+    # For cross repo pull request, we need to send head parameter like fork_owner:branch as per git documentation here : https://docs.github.com/en/rest/pulls/pulls?apiVersion=2022-11-28#create-a-pull-request
+    # head parameter usage : The name of the branch where your changes are implemented. For cross-repository pull requests in the same network, namespace head with a user like this: username:branch.
+    if fork_owner and platform == ProviderType.GITHUB:
+        head_branch = f'{fork_owner}:{branch_name}'
+    else:
+        head_branch = branch_name
     # If we are not sending a PR, we can finish early and return the
     # URL for the user to open a PR manually
     if pr_type == 'branch':
-        url = f'https://github.com/{push_owner}/{github_issue.repo}/compare/{branch_name}?expand=1'
+        url = handler.get_compare_url(branch_name)
     else:
         # Prepare the PR for the GitHub API
         data = {
-            'title': pr_title,  # No need to escape title for GitHub API
-            'body': pr_body,
-            'head': branch_name,
-            'base': base_branch,
+            'title': final_pr_title,
+            ('body' if platform == ProviderType.GITHUB else 'description'): pr_body,
+            (
+                'head' if platform == ProviderType.GITHUB else 'source_branch'
+            ): head_branch,
+            (
+                'base' if platform == ProviderType.GITHUB else 'target_branch'
+            ): base_branch,
             'draft': pr_type == 'draft',
         }
 
-        # Send the PR and get its URL to tell the user
-        response = requests.post(f'{base_url}/pulls', headers=headers, json=data)
-        if response.status_code == 403:
-            raise RuntimeError(
-                'Failed to create pull request due to missing permissions. '
-                'Make sure that the provided token has push permissions for the repository.'
-            )
-        response.raise_for_status()
-        pr_data = response.json()
+        pr_data = handler.create_pull_request(data)
+        url = pr_data['html_url']
 
         # Request review if a reviewer was specified
         if reviewer and pr_type != 'branch':
-            review_data = {'reviewers': [reviewer]}
-            review_response = requests.post(
-                f'{base_url}/pulls/{pr_data["number"]}/requested_reviewers',
-                headers=headers,
-                json=review_data,
-            )
-            if review_response.status_code != 201:
-                print(
-                    f'Warning: Failed to request review from {reviewer}: {review_response.text}'
-                )
+            number = pr_data['number']
+            handler.request_reviewers(reviewer, number)
 
-        url = pr_data['html_url']
-
-    print(f'{pr_type} created: {url}\n\n--- Title: {pr_title}\n\n--- Body:\n{pr_body}')
+    logger.info(
+        f'{pr_type} created: {url}\n\n--- Title: {final_pr_title}\n\n--- Body:\n{pr_body}'
+    )
 
     return url
 
 
-def reply_to_comment(github_token: str, comment_id: str, reply: str):
-    """Reply to a comment on a GitHub issue or pull request.
-
-    Args:
-        github_token: The GitHub token to use for authentication
-        comment_id: The ID of the comment to reply to
-        reply: The reply message to post
-    """
-    # Opting for graphql as REST API doesn't allow reply to replies in comment threads
-    query = """
-            mutation($body: String!, $pullRequestReviewThreadId: ID!) {
-                addPullRequestReviewThreadReply(input: { body: $body, pullRequestReviewThreadId: $pullRequestReviewThreadId }) {
-                    comment {
-                        id
-                        body
-                        createdAt
-                    }
-                }
-            }
-            """
-
-    # Prepare the reply to the comment
-    comment_reply = f'Openhands fix success summary\n\n\n{reply}'
-    variables = {'body': comment_reply, 'pullRequestReviewThreadId': comment_id}
-    url = 'https://api.github.com/graphql'
-    headers = {
-        'Authorization': f'Bearer {github_token}',
-        'Content-Type': 'application/json',
-    }
-
-    # Send the reply to the comment
-    response = requests.post(
-        url, json={'query': query, 'variables': variables}, headers=headers
-    )
-    response.raise_for_status()
-
-
-def send_comment_msg(base_url: str, issue_number: int, github_token: str, msg: str):
-    """Send a comment message to a GitHub issue or pull request.
-
-    Args:
-        base_url: The base URL of the GitHub repository API
-        issue_number: The issue or pull request number
-        github_token: The GitHub token to use for authentication
-        msg: The message content to post as a comment
-    """
-    # Set up headers for GitHub API
-    headers = {
-        'Authorization': f'token {github_token}',
-        'Accept': 'application/vnd.github.v3+json',
-    }
-
-    # Post a comment on the PR
-    comment_url = f'{base_url}/issues/{issue_number}/comments'
-    comment_data = {'body': msg}
-    comment_response = requests.post(comment_url, headers=headers, json=comment_data)
-    if comment_response.status_code != 201:
-        print(
-            f'Failed to post comment: {comment_response.status_code} {comment_response.text}'
-        )
-    else:
-        print(f'Comment added to the PR: {msg}')
-
-
 def update_existing_pull_request(
-    github_issue: GithubIssue,
-    github_token: str,
-    github_username: str | None,
+    issue: Issue,
+    token: str,
+    username: str | None,
+    platform: ProviderType,
     patch_dir: str,
     llm_config: LLMConfig,
     comment_message: str | None = None,
     additional_message: str | None = None,
+    base_domain: str | None = None,
 ) -> str:
     """Update an existing pull request with the new patches.
 
     Args:
-        github_issue: The issue to update.
-        github_token: The GitHub token to use for authentication.
-        github_username: The GitHub username to use for authentication.
+        issue: The issue to update.
+        token: The  token to use for authentication.
+        username: The username to use for authentication.
+        platform: The platform of the repository.
         patch_dir: The directory containing the patches to apply.
         llm_config: The LLM configuration to use for summarizing changes.
         comment_message: The main message to post as a comment on the PR.
         additional_message: The additional messages to post as a comment on the PR in json list format.
+        base_domain: The base domain for the git server (defaults to "github.com" for GitHub and "gitlab.com" for GitLab)
     """
-    # Set up base URL for GitHub API
-    base_url = f'https://api.github.com/repos/{github_issue.owner}/{github_issue.repo}'
-    branch_name = github_issue.head_branch
+    # Set up headers and base URL for GitHub or GitLab API
+
+    # Determine default base_domain based on platform
+    if base_domain is None:
+        base_domain = 'github.com' if platform == ProviderType.GITHUB else 'gitlab.com'
+
+    handler = None
+    if platform == ProviderType.GITHUB:
+        handler = ServiceContextIssue(
+            GithubIssueHandler(issue.owner, issue.repo, token, username, base_domain),
+            llm_config,
+        )
+    else:  # platform == Platform.GITLAB
+        handler = ServiceContextIssue(
+            GitlabIssueHandler(issue.owner, issue.repo, token, username, base_domain),
+            llm_config,
+        )
+
+    branch_name = issue.head_branch
 
     # Prepare the push command
     push_command = (
         f'git -C {patch_dir} push '
-        f'https://{github_username}:{github_token}@github.com/'
-        f'{github_issue.owner}/{github_issue.repo}.git {branch_name}'
+        f'{handler.get_authorize_url()}'
+        f'{issue.owner}/{issue.repo}.git {branch_name}'
     )
 
     # Push the changes to the existing branch
     result = subprocess.run(push_command, shell=True, capture_output=True, text=True)
     if result.returncode != 0:
-        print(f'Error pushing changes: {result.stderr}')
+        logger.error(f'Error pushing changes: {result.stderr}')
         raise RuntimeError('Failed to push changes to the remote repository')
 
-    pr_url = f'https://github.com/{github_issue.owner}/{github_issue.repo}/pull/{github_issue.number}'
-    print(f'Updated pull request {pr_url} with new patches.')
+    pr_url = handler.get_pull_url(issue.number)
+    logger.info(f'Updated pull request {pr_url} with new patches.')
 
     # Generate a summary of all comment success indicators for PR message
     if not comment_message and additional_message:
@@ -508,18 +473,18 @@ def update_existing_pull_request(
 
     # Post a comment on the PR
     if comment_message:
-        send_comment_msg(base_url, github_issue.number, github_token, comment_message)
+        handler.send_comment_msg(issue.number, comment_message)
 
     # Reply to each unresolved comment thread
-    if additional_message and github_issue.thread_ids:
+    if additional_message and issue.thread_ids:
         try:
             explanations = json.loads(additional_message)
             for count, reply_comment in enumerate(explanations):
-                comment_id = github_issue.thread_ids[count]
-                reply_to_comment(github_token, comment_id, reply_comment)
+                comment_id = issue.thread_ids[count]
+                handler.reply_to_comment(issue.number, comment_id, reply_comment)
         except (json.JSONDecodeError, TypeError):
-            msg = f'Error occured when replying to threads; success explanations {additional_message}'
-            send_comment_msg(base_url, github_issue.number, github_token, msg)
+            msg = f'Error occurred when replying to threads; success explanations {additional_message}'
+            handler.send_comment_msg(issue.number, msg)
 
     return pr_url
 
@@ -527,17 +492,23 @@ def update_existing_pull_request(
 def process_single_issue(
     output_dir: str,
     resolver_output: ResolverOutput,
-    github_token: str,
-    github_username: str,
+    token: str,
+    username: str,
+    platform: ProviderType,
     pr_type: str,
     llm_config: LLMConfig,
     fork_owner: str | None,
     send_on_failure: bool,
     target_branch: str | None = None,
     reviewer: str | None = None,
+    pr_title: str | None = None,
+    base_domain: str | None = None,
 ) -> None:
+    # Determine default base_domain based on platform
+    if base_domain is None:
+        base_domain = 'github.com' if platform == ProviderType.GITHUB else 'gitlab.com'
     if not resolver_output.success and not send_on_failure:
-        print(
+        logger.info(
             f'Issue {resolver_output.issue.number} was not successfully resolved. Skipping PR creation.'
         )
         return
@@ -567,65 +538,53 @@ def process_single_issue(
 
     if issue_type == 'pr':
         update_existing_pull_request(
-            github_issue=resolver_output.issue,
-            github_token=github_token,
-            github_username=github_username,
+            issue=resolver_output.issue,
+            token=token,
+            username=username,
+            platform=platform,
             patch_dir=patched_repo_dir,
-            additional_message=resolver_output.success_explanation,
+            additional_message=resolver_output.result_explanation,
             llm_config=llm_config,
+            base_domain=base_domain,
         )
     else:
         send_pull_request(
-            github_issue=resolver_output.issue,
-            github_token=github_token,
-            github_username=github_username,
+            issue=resolver_output.issue,
+            token=token,
+            username=username,
+            platform=platform,
             patch_dir=patched_repo_dir,
             pr_type=pr_type,
             fork_owner=fork_owner,
-            additional_message=resolver_output.success_explanation,
+            additional_message=resolver_output.result_explanation,
             target_branch=target_branch,
             reviewer=reviewer,
+            pr_title=pr_title,
+            base_domain=base_domain,
         )
 
 
-def process_all_successful_issues(
-    output_dir: str,
-    github_token: str,
-    github_username: str,
-    pr_type: str,
-    llm_config: LLMConfig,
-    fork_owner: str | None,
-) -> None:
-    output_path = os.path.join(output_dir, 'output.jsonl')
-    for resolver_output in load_all_resolver_outputs(output_path):
-        if resolver_output.success:
-            print(f'Processing issue {resolver_output.issue.number}')
-            process_single_issue(
-                output_dir,
-                resolver_output,
-                github_token,
-                github_username,
-                pr_type,
-                llm_config,
-                fork_owner,
-                False,
-                None,
-            )
-
-
-def main():
-    parser = argparse.ArgumentParser(description='Send a pull request to Github.')
-    parser.add_argument(
-        '--github-token',
-        type=str,
-        default=None,
-        help='Github token to access the repository.',
+def main() -> None:
+    parser = argparse.ArgumentParser(
+        description='Send a pull request to Github or Gitlab.'
     )
     parser.add_argument(
-        '--github-username',
+        '--selected-repo',
         type=str,
         default=None,
-        help='Github username to access the repository.',
+        help='repository to send pull request in form of `owner/repo`.',
+    )
+    parser.add_argument(
+        '--token',
+        type=str,
+        default=None,
+        help='token to access the repository.',
+    )
+    parser.add_argument(
+        '--username',
+        type=str,
+        default=None,
+        help='username to access the repository.',
     )
     parser.add_argument(
         '--output-dir',
@@ -684,64 +643,69 @@ def main():
     parser.add_argument(
         '--reviewer',
         type=str,
-        help='GitHub username of the person to request review from',
+        help='GitHub or GitLab username of the person to request review from',
         default=None,
     )
+    parser.add_argument(
+        '--pr-title',
+        type=str,
+        help='Custom title for the pull request',
+        default=None,
+    )
+    parser.add_argument(
+        '--base-domain',
+        type=str,
+        default=None,
+        help='Base domain for the git server (defaults to "github.com" for GitHub and "gitlab.com" for GitLab)',
+    )
     my_args = parser.parse_args()
 
-    github_token = (
-        my_args.github_token if my_args.github_token else os.getenv('GITHUB_TOKEN')
-    )
-    if not github_token:
+    token = my_args.token or os.getenv('GITHUB_TOKEN') or os.getenv('GITLAB_TOKEN')
+    if not token:
         raise ValueError(
-            'Github token is not set, set via --github-token or GITHUB_TOKEN environment variable.'
+            'token is not set, set via --token or GITHUB_TOKEN or GITLAB_TOKEN environment variable.'
         )
-    github_username = (
-        my_args.github_username
-        if my_args.github_username
-        else os.getenv('GITHUB_USERNAME')
+    username = my_args.username if my_args.username else os.getenv('GIT_USERNAME')
+
+    platform = call_async_from_sync(
+        identify_token,
+        GENERAL_TIMEOUT,
+        token,
+        my_args.base_domain,
     )
 
+    api_key = my_args.llm_api_key or os.environ['LLM_API_KEY']
     llm_config = LLMConfig(
         model=my_args.llm_model or os.environ['LLM_MODEL'],
-        api_key=my_args.llm_api_key or os.environ['LLM_API_KEY'],
+        api_key=SecretStr(api_key) if api_key else None,
         base_url=my_args.llm_base_url or os.environ.get('LLM_BASE_URL', None),
     )
 
     if not os.path.exists(my_args.output_dir):
         raise ValueError(f'Output directory {my_args.output_dir} does not exist.')
 
-    if my_args.issue_number == 'all_successful':
-        if not github_username:
-            raise ValueError('Github username is required.')
-        process_all_successful_issues(
-            my_args.output_dir,
-            github_token,
-            github_username,
-            my_args.pr_type,
-            llm_config,
-            my_args.fork_owner,
-        )
-    else:
-        if not my_args.issue_number.isdigit():
-            raise ValueError(f'Issue number {my_args.issue_number} is not a number.')
-        issue_number = int(my_args.issue_number)
-        output_path = os.path.join(my_args.output_dir, 'output.jsonl')
-        resolver_output = load_single_resolver_output(output_path, issue_number)
-        if not github_username:
-            raise ValueError('Github username is required.')
-        process_single_issue(
-            my_args.output_dir,
-            resolver_output,
-            github_token,
-            github_username,
-            my_args.pr_type,
-            llm_config,
-            my_args.fork_owner,
-            my_args.send_on_failure,
-            my_args.target_branch,
-            my_args.reviewer,
-        )
+    if not my_args.issue_number.isdigit():
+        raise ValueError(f'Issue number {my_args.issue_number} is not a number.')
+    issue_number = int(my_args.issue_number)
+    output_path = os.path.join(my_args.output_dir, 'output.jsonl')
+    resolver_output = load_single_resolver_output(output_path, issue_number)
+    if not username:
+        raise ValueError('username is required.')
+    process_single_issue(
+        my_args.output_dir,
+        resolver_output,
+        token,
+        username,
+        platform,
+        my_args.pr_type,
+        llm_config,
+        my_args.fork_owner,
+        my_args.send_on_failure,
+        my_args.target_branch,
+        my_args.reviewer,
+        my_args.pr_title,
+        my_args.base_domain,
+    )
 
 
 if __name__ == '__main__':
